
#ifndef VIBESETTINGS_EXPORT_H
#define VIBESETTINGS_EXPORT_H

#ifdef VIBESETTINGS_STATIC_DEFINE
#  define VIBESETTINGS_EXPORT
#  define VIBESETTINGS_NO_EXPORT
#else
#  ifndef VIBESETTINGS_EXPORT
#    ifdef VibeSettings_EXPORTS
        /* We are building this library */
#      define VIBESETTINGS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VIBESETTINGS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VIBESETTINGS_NO_EXPORT
#    define VIBESETTINGS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VIBESETTINGS_DEPRECATED
#  define VIBESETTINGS_DEPRECATED 
#endif

#ifndef VIBESETTINGS_DEPRECATED_EXPORT
#  define VIBESETTINGS_DEPRECATED_EXPORT VIBESETTINGS_EXPORT VIBESETTINGS_DEPRECATED
#endif

#ifndef VIBESETTINGS_DEPRECATED_NO_EXPORT
#  define VIBESETTINGS_DEPRECATED_NO_EXPORT VIBESETTINGS_NO_EXPORT VIBESETTINGS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VIBESETTINGS_NO_DEPRECATED
#    define VIBESETTINGS_NO_DEPRECATED
#  endif
#endif

#endif
