#ifndef VIBE_VERSION_H
#define VIBE_VERSION_H

#define VIBE_VERSION_STRING "0.9.0"
#define VIBE_VERSION_MAJOR 0
#define VIBE_VERSION_MINOR 9
#define VIBE_VERSION_PATCH 0
#define VIBE_VERSION ((0<<16)|(9<<8)|(0))

#endif
