
#ifndef VIBECORE_EXPORT_H
#define VIBECORE_EXPORT_H

#ifdef VIBECORE_STATIC_DEFINE
#  define VIBECORE_EXPORT
#  define VIBECORE_NO_EXPORT
#else
#  ifndef VIBECORE_EXPORT
#    ifdef VibeCore_EXPORTS
        /* We are building this library */
#      define VIBECORE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VIBECORE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VIBECORE_NO_EXPORT
#    define VIBECORE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VIBECORE_DEPRECATED
#  define VIBECORE_DEPRECATED 
#endif

#ifndef VIBECORE_DEPRECATED_EXPORT
#  define VIBECORE_DEPRECATED_EXPORT VIBECORE_EXPORT VIBECORE_DEPRECATED
#endif

#ifndef VIBECORE_DEPRECATED_NO_EXPORT
#  define VIBECORE_DEPRECATED_NO_EXPORT VIBECORE_NO_EXPORT VIBECORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VIBECORE_NO_DEPRECATED
#    define VIBECORE_NO_DEPRECATED
#  endif
#endif

#endif
